"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  getBabelUtils: () => getBabelUtils,
  pluginBabel: () => pluginBabel
});
module.exports = __toCommonJS(src_exports);

// src/plugin.ts
var import_shared2 = require("@rsbuild/shared");
var import_lodash = require("lodash");

// src/helper.ts
var import_path = require("path");
var import_shared = require("@rsbuild/shared");
var import_upath = __toESM(require("upath"));
var normalizeToPosixPath = (p) => import_upath.default.normalizeSafe((0, import_path.normalize)(p || "")).replace(/^([a-zA-Z]+):/, (_, m) => `/${m.toLowerCase()}`);
var formatPath = (originPath) => {
  if ((0, import_path.isAbsolute)(originPath)) {
    return originPath.split(import_path.sep).join("/");
  }
  return originPath;
};
var getPluginItemName = (item) => {
  if (typeof item === "string") {
    return formatPath(item);
  }
  if (Array.isArray(item) && typeof item[0] === "string") {
    return formatPath(item[0]);
  }
  return null;
};
var addPlugins = (plugins, config) => {
  if (config.plugins) {
    config.plugins.push(...plugins);
  } else {
    config.plugins = plugins;
  }
};
var addPresets = (presets, config) => {
  if (config.presets) {
    config.presets.push(...presets);
  } else {
    config.presets = presets;
  }
};
var removePlugins = (plugins, config) => {
  if (!config.plugins) {
    return;
  }
  const removeList = (0, import_shared.castArray)(plugins);
  config.plugins = config.plugins.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var removePresets = (presets, config) => {
  if (!config.presets) {
    return;
  }
  const removeList = (0, import_shared.castArray)(presets);
  config.presets = config.presets.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var modifyPresetOptions = (presetName, options, presets = []) => {
  presets.forEach((preset, index) => {
    if (Array.isArray(preset)) {
      if (typeof preset[0] === "string" && normalizeToPosixPath(preset[0]).includes(presetName)) {
        preset[1] = {
          ...preset[1] || {},
          ...options
          // `options` is specific to different presets
        };
      }
    } else if (typeof preset === "string" && normalizeToPosixPath(preset).includes(presetName)) {
      presets[index] = [preset, options];
    }
  });
};
var getBabelUtils = (config) => {
  const noop = () => {
  };
  return {
    addPlugins: (plugins) => addPlugins(plugins, config),
    addPresets: (presets) => addPresets(presets, config),
    removePlugins: (plugins) => removePlugins(plugins, config),
    removePresets: (presets) => removePresets(presets, config),
    // `addIncludes` and `addExcludes` are noop functions by default,
    // It can be overridden by `extraBabelUtils`.
    addIncludes: noop,
    addExcludes: noop,
    // Compat `presetEnvOptions` and `presetReactOptions` in Eden.
    modifyPresetEnvOptions: (options) => modifyPresetOptions("@babel/preset-env", options, config.presets || []),
    modifyPresetReactOptions: (options) => modifyPresetOptions("@babel/preset-react", options, config.presets || [])
  };
};
var applyUserBabelConfig = (defaultOptions, userBabelConfig, extraBabelUtils) => {
  if (userBabelConfig) {
    const babelUtils = {
      ...getBabelUtils(defaultOptions),
      ...extraBabelUtils
    };
    return (0, import_shared.mergeChainedOptions)({
      defaults: defaultOptions,
      options: userBabelConfig,
      utils: babelUtils
    });
  }
  return defaultOptions;
};

// src/plugin.ts
var DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS = {
  allowNamespaces: true,
  allExtensions: true,
  allowDeclareFields: true,
  // aligns Babel's behavior with TypeScript's default behavior.
  // https://babeljs.io/docs/en/babel-preset-typescript#optimizeconstenums
  optimizeConstEnums: true,
  isTSX: true
};
var pluginBabel = (options = {}) => ({
  name: "rsbuild:babel",
  setup(api) {
    if (api.context.bundlerType === "webpack") {
      return;
    }
    api.modifyBundlerChain(async (chain, { CHAIN_ID, isProd }) => {
      const getBabelOptions = () => {
        const includes2 = [];
        const excludes2 = [];
        const babelUtils = {
          addIncludes(items) {
            if (Array.isArray(items)) {
              includes2.push(...items);
            } else {
              includes2.push(items);
            }
          },
          addExcludes(items) {
            if (Array.isArray(items)) {
              excludes2.push(...items);
            } else {
              excludes2.push(items);
            }
          }
        };
        const baseConfig = {
          plugins: [],
          presets: [
            // TODO: only apply preset-typescript for ts file (isTSX & allExtensions false)
            [
              require.resolve("@babel/preset-typescript"),
              DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS
            ]
          ]
        };
        const userBabelConfig = applyUserBabelConfig(
          (0, import_lodash.cloneDeep)(baseConfig),
          options,
          babelUtils
        );
        const babelOptions2 = {
          babelrc: false,
          configFile: false,
          compact: isProd,
          ...userBabelConfig
        };
        return {
          babelOptions: babelOptions2,
          includes: includes2,
          excludes: excludes2
        };
      };
      const { babelOptions, includes = [], excludes = [] } = getBabelOptions();
      const rule = chain.module.rule(CHAIN_ID.RULE.JS);
      includes.forEach((condition) => {
        rule.include.add(condition);
      });
      excludes.forEach((condition) => {
        rule.exclude.add(condition);
      });
      rule.test(import_shared2.SCRIPT_REGEX).use(CHAIN_ID.USE.BABEL).after(CHAIN_ID.USE.SWC).loader(require.resolve("babel-loader")).options(babelOptions);
    });
  }
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getBabelUtils,
  pluginBabel
});
