var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});

// ../../node_modules/.pnpm/@modern-js+module-tools@2.41.0_typescript@5.3.2/node_modules/@modern-js/module-tools/shims/esm.js
import { fileURLToPath } from "url";
import path from "path";

// ../../scripts/requireShims.js
import { createRequire } from "module";
global.require = createRequire(import.meta.url);

// src/plugin.ts
import { SCRIPT_REGEX } from "@rsbuild/shared";
import { cloneDeep } from "lodash";

// src/helper.ts
import { isAbsolute, normalize, sep } from "path";
import { castArray, mergeChainedOptions } from "@rsbuild/shared";
import upath from "upath";
var normalizeToPosixPath = (p) => upath.normalizeSafe(normalize(p || "")).replace(/^([a-zA-Z]+):/, (_, m) => `/${m.toLowerCase()}`);
var formatPath = (originPath) => {
  if (isAbsolute(originPath)) {
    return originPath.split(sep).join("/");
  }
  return originPath;
};
var getPluginItemName = (item) => {
  if (typeof item === "string") {
    return formatPath(item);
  }
  if (Array.isArray(item) && typeof item[0] === "string") {
    return formatPath(item[0]);
  }
  return null;
};
var addPlugins = (plugins, config) => {
  if (config.plugins) {
    config.plugins.push(...plugins);
  } else {
    config.plugins = plugins;
  }
};
var addPresets = (presets, config) => {
  if (config.presets) {
    config.presets.push(...presets);
  } else {
    config.presets = presets;
  }
};
var removePlugins = (plugins, config) => {
  if (!config.plugins) {
    return;
  }
  const removeList = castArray(plugins);
  config.plugins = config.plugins.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var removePresets = (presets, config) => {
  if (!config.presets) {
    return;
  }
  const removeList = castArray(presets);
  config.presets = config.presets.filter((item) => {
    const name = getPluginItemName(item);
    if (name) {
      return !removeList.find((removeItem) => name.includes(removeItem));
    }
    return true;
  });
};
var modifyPresetOptions = (presetName, options, presets = []) => {
  presets.forEach((preset, index) => {
    if (Array.isArray(preset)) {
      if (typeof preset[0] === "string" && normalizeToPosixPath(preset[0]).includes(presetName)) {
        preset[1] = {
          ...preset[1] || {},
          ...options
          // `options` is specific to different presets
        };
      }
    } else if (typeof preset === "string" && normalizeToPosixPath(preset).includes(presetName)) {
      presets[index] = [preset, options];
    }
  });
};
var getBabelUtils = (config) => {
  const noop = () => {
  };
  return {
    addPlugins: (plugins) => addPlugins(plugins, config),
    addPresets: (presets) => addPresets(presets, config),
    removePlugins: (plugins) => removePlugins(plugins, config),
    removePresets: (presets) => removePresets(presets, config),
    // `addIncludes` and `addExcludes` are noop functions by default,
    // It can be overridden by `extraBabelUtils`.
    addIncludes: noop,
    addExcludes: noop,
    // Compat `presetEnvOptions` and `presetReactOptions` in Eden.
    modifyPresetEnvOptions: (options) => modifyPresetOptions("@babel/preset-env", options, config.presets || []),
    modifyPresetReactOptions: (options) => modifyPresetOptions("@babel/preset-react", options, config.presets || [])
  };
};
var applyUserBabelConfig = (defaultOptions, userBabelConfig, extraBabelUtils) => {
  if (userBabelConfig) {
    const babelUtils = {
      ...getBabelUtils(defaultOptions),
      ...extraBabelUtils
    };
    return mergeChainedOptions({
      defaults: defaultOptions,
      options: userBabelConfig,
      utils: babelUtils
    });
  }
  return defaultOptions;
};

// src/plugin.ts
var DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS = {
  allowNamespaces: true,
  allExtensions: true,
  allowDeclareFields: true,
  // aligns Babel's behavior with TypeScript's default behavior.
  // https://babeljs.io/docs/en/babel-preset-typescript#optimizeconstenums
  optimizeConstEnums: true,
  isTSX: true
};
var pluginBabel = (options = {}) => ({
  name: "rsbuild:babel",
  setup(api) {
    if (api.context.bundlerType === "webpack") {
      return;
    }
    api.modifyBundlerChain(async (chain, { CHAIN_ID, isProd }) => {
      const getBabelOptions = () => {
        const includes2 = [];
        const excludes2 = [];
        const babelUtils = {
          addIncludes(items) {
            if (Array.isArray(items)) {
              includes2.push(...items);
            } else {
              includes2.push(items);
            }
          },
          addExcludes(items) {
            if (Array.isArray(items)) {
              excludes2.push(...items);
            } else {
              excludes2.push(items);
            }
          }
        };
        const baseConfig = {
          plugins: [],
          presets: [
            // TODO: only apply preset-typescript for ts file (isTSX & allExtensions false)
            [
              __require.resolve("@babel/preset-typescript"),
              DEFAULT_BABEL_PRESET_TYPESCRIPT_OPTIONS
            ]
          ]
        };
        const userBabelConfig = applyUserBabelConfig(
          cloneDeep(baseConfig),
          options,
          babelUtils
        );
        const babelOptions2 = {
          babelrc: false,
          configFile: false,
          compact: isProd,
          ...userBabelConfig
        };
        return {
          babelOptions: babelOptions2,
          includes: includes2,
          excludes: excludes2
        };
      };
      const { babelOptions, includes = [], excludes = [] } = getBabelOptions();
      const rule = chain.module.rule(CHAIN_ID.RULE.JS);
      includes.forEach((condition) => {
        rule.include.add(condition);
      });
      excludes.forEach((condition) => {
        rule.exclude.add(condition);
      });
      rule.test(SCRIPT_REGEX).use(CHAIN_ID.USE.BABEL).after(CHAIN_ID.USE.SWC).loader(__require.resolve("babel-loader")).options(babelOptions);
    });
  }
});
export {
  getBabelUtils,
  pluginBabel
};
